/*******************************************************************************
 * Copyright (c) 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.ui.internal.presentations.newapi;

import org.eclipse.jface.util.Geometry;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.presentations.StackDropResult;

/**
 * @since 3.0
 */
public class ReplaceDragHandler extends TabDragHandler {

    private final class DragCookie {
        int insertPosition;

        public DragCookie(int pos) {
            insertPosition = pos;
        }
    };

    private AbstractTabFolder tabFolder;

    /* (non-Javadoc)
     * @see org.eclipse.ui.internal.presentations.newapi.TabDragHandler#dragOver(org.eclipse.swt.widgets.Control, org.eclipse.swt.graphics.Point)
     */
    public StackDropResult dragOver(Control currentControl, Point location,
            int dragStart) {

        // Determine which tab we're currently dragging over
        Point localPos = tabFolder.getControl().toControl(location);

        AbstractTabItem tabUnderPointer = tabFolder.getItem(localPos);

        // This drop target only deals with tabs... if we're not dragging over
        // a tab, exit.
        if (tabUnderPointer == null) {
            Rectangle titleArea = tabFolder.getTabArea();

            // If we're dragging over the title area, treat this as a drop in the last
            // tab position.
            if (titleArea.contains(localPos)) {
                int dragOverIndex = tabFolder.getItemCount();
                AbstractTabItem lastTab = tabFolder.getItem(dragOverIndex - 1);

                // Can't drag to end unless you can see the end
                if (!lastTab.isShowing()) {
                    return null;
                }

                if (dragStart >= 0) {
                    dragOverIndex--;

                    return new StackDropResult(Geometry.toDisplay(tabFolder
                            .getControl(), lastTab.getBounds()), new Integer(
                            dragOverIndex));
                }

                // Make the drag-over rectangle look like a tab at the end of the tab region.
                // We don't actually know how wide the tab will be when it's dropped, so just
                // make it 3 times wider than it is tall.
                Rectangle dropRectangle = Geometry.toDisplay(tabFolder
                        .getControl(), titleArea);

                dropRectangle.width = 3 * dropRectangle.height;
                return new StackDropResult(dropRectangle, new Integer(
                        dragOverIndex));

            } else {
                return null;
            }
        }

        if (!tabUnderPointer.isShowing()) {
            return null;
        }

        return new StackDropResult(Geometry.toDisplay(tabFolder.getControl(),
                tabUnderPointer.getBounds()), new DragCookie(tabFolder
                .indexOf(tabUnderPointer)));
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.internal.presentations.newapi.TabDragHandler#getInsertionPosition(java.lang.Object)
     */
    public int getInsertionPosition(Object cookie) {
        if (cookie instanceof DragCookie) {
            return Math.min(tabFolder.getItemCount(),
                    ((DragCookie) cookie).insertPosition);
        }

        return tabFolder.getItemCount();
    }

}